@@ -37,13 +37,14 @@ makeFrom = derivationCustom "From" $ \(_,d) -> Right $ concatMap (makeFromCtor d
 
 
 makeFromCtor :: DataDecl -> CtorDecl -> [Decl]
-makeFromCtor d c = [TypeSig sl [name from] typ, FunBind $ match : [defMatch | length (dataDeclCtors d) > 1]]
+makeFromCtor d c | isIdent n = [TypeSig sl [name from] typ, FunBind $ match : [defMatch | length (dataDeclCtors d) > 1]]
+                 | otherwise = []
     where
         n = ctorDeclName c
         from = "from" ++ n
 
         typ = TyFun (dataDeclType d)
-            (tyTuple $ map (fromBangType . snd) $ ctorDeclFields c)
+            (tyTuple $ map snd $ ctorDeclFields c)
 
         match = Match sl (name from) [pat] Nothing (UnGuardedRhs rhs) (BDecls [])
         pat = (length vars == 0 ? id $ PParen) $ PApp (qname n) (map pVar vars)
